/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.fluxml;

import java.io.IOException;
import java.util.Map;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.joda.time.LocalTime;

/**
 * MarkupWriter provides an API for XML serialization
 * 
 * @author tiwe
 * @version $Id$
 */
public interface MarkupWriter<A extends MarkupWriter<A>> {
    
    /**
     * @param name
     * @param value
     * @return
     * @throws IOException
     */
    A attribute(String name, String value) throws IOException;
    
    /**
     * @param name
     * @param value
     * @return
     * @throws IOException
     */
    A attribute(String name, DateTime value) throws IOException;
    
    /**
     * @param name
     * @param value
     * @return
     * @throws IOException
     */
    A attribute(String name, LocalDate value) throws IOException;
    
    /**
     * @param name
     * @param value
     * @return
     * @throws IOException
     */
    A attribute(String name, LocalTime value) throws IOException;
    
    /**
     * @param attributes
     * @return
     * @throws IOException
     */
    A attributes(Map<String, String> attributes) throws IOException;

    /**
     * @param name
     * @return
     * @throws IOException
     */
    A begin(String name) throws IOException;

    /**
     * @param name
     * @param content
     * @return
     * @throws IOException 
     */
    A element(String name) throws IOException;

    /**
     * @param name
     * @param content
     * @return
     * @throws IOException 
     */
    A element(String name, String content) throws IOException;

    /**
     * @param name
     * @param content
     * @return
     * @throws IOException
     */
    A element(String name, DateTime content) throws IOException;
    
    /**
     * @param name
     * @param content
     * @return
     * @throws IOException
     */
    A element(String name, LocalDate content) throws IOException;
    
    /**
     * @param name
     * @param content
     * @return
     * @throws IOException
     */
    A element(String name, LocalTime content) throws IOException;
    
    /**
     * @param name
     * @return
     * @throws IOException
     */
    A end(String name) throws IOException;

    /**
     * @param text
     * @return
     * @throws IOException
     */
    A print(String text) throws IOException;

    /**
     * @param text
     * @return
     * @throws IOException
     */
    A printRaw(String text) throws IOException;

}