/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.fluxml.tree;

import java.io.IOException;

import com.mysema.commons.fluxml.MarkupWriter;

/**
 * AbstractTreeRenderer provides
 * 
 * @author tiwe
 * @version $Id$
 */
public abstract class TreeRenderer<NodeType, Writer extends MarkupWriter<Writer>> {

    protected boolean showRoot;

    public void render(TreeModel<NodeType> model, Writer writer)
            throws IOException {
        beginList(writer);
        if (showRoot) {
            renderNode(model, model.getRoot(), writer);
        } else {
            for (NodeType child : model.getChildren(model.getRoot())) {
                renderNode(model, child, writer);
            }
        }
        endList(writer);
    }

    protected void renderNode(TreeModel<NodeType> model, NodeType node,
            Writer writer) throws IOException {
        beginItem(writer);
        renderItemContent(writer, node);
        if (!model.isLeaf(node)) {
            beginList(writer);
            for (NodeType child : model.getChildren(node)) {
                renderNode(model, child, writer);
            }
            endList(writer);
        }
        endItem(writer);
    }

    protected abstract void beginItem(Writer writer) throws IOException;

    protected abstract void beginList(Writer writer) throws IOException;

    protected abstract void endItem(Writer writer) throws IOException;

    protected abstract void endList(Writer writer) throws IOException;

    protected abstract void renderItemContent(Writer writer, NodeType node)
            throws IOException;

}
